/*****************************************************************************
 *
 *   (C) Copyright Southsoftware.com, 2005
 *
 *   Title: XMLDLG.H
 *
 *   Version:   1.09
 *
 *   Date:  18-Sep-2005
 *
 *   Author:    Master
 *
 *----------------------------------------------------------------------------
 *
 *   Change log:
 *
 *   DATE        DESCRIPTION
 *   ----------- -------------------------------------------------------------
 *   18-Sep-2005 Original
 *   12-Jan-2009 Last edition
 */

#ifndef XMLDLG_H_
#define XMLDLG_H_ 1

///////////////////////////////////////////////////////////////////////////////
// Decorate exportable functions for Win32 DLL linking.
#if (defined(WIN32) || defined(_WIN32)) && !defined(APPLOC_STATICLIB)
#if defined(APPLOC_BUILDDLL)
#define APPLOC_EXTERN  __declspec(dllexport)
#else
#define APPLOC_EXTERN  __declspec(dllimport)
#endif
#else
#define APPLOC_EXTERN
#endif

///////////////////////////////////////////////////////////////////////////////
#ifdef _MSC_VER
// Assume 4 byte packing throughout.
#include <pshpack4.h>
#elif __BORLANDC__
// Turn off "Structure packing size has changed" warning message.
#pragma option -w-pck
// Assume 4 byte packing throughout.
#pragma option -b.
#include <pshpack4.h>
#pragma option -b
#else
#error You must 4-byte align these structures with the appropriate compiler directives.
#endif

#ifdef __cplusplus
extern "C" {
#endif

///////////////////////////////////////////////////////////////////////////////
typedef struct tagXMLRECT {
    int top;
    int left;
    int right;
    int bottom;
} XMLRECT, *LPXMLRECT;

///////////////////////////////////////////////////////////////////////////////
typedef enum {
    XMLDLGPARSE_NOERR = 0, // no error
    XMLDLGPARSE_UNEXPDLG,  // unexpected "dialog" tag
    XMLDLGPARSE_UNCLOSED,  // unclosed tag found
} XMLDLGPARSE_E;

typedef struct tagGETXMLDLGINFO {
    int nPanelLeft;            // the position of the horizontal scroll box if the dialog has scroll bars
    int nPanelTop;             // the position of the vertical scroll box if the dialog has scroll bars
    unsigned int nPanelWidth;  // the width, not scaled, of the client area of the dialog
    unsigned int nPanelHeight; // the height, not scaled, of the client area of the dialog
    XMLDLGPARSE_E parseerror;  // the last error in parsing of XML data
    unsigned int nPanelRects;  // the number of items in the lpPanelRects
    XMLRECT *lpPanelRects;     // the list of XMLRECT structures for all cell with storerect
} GETXMLDLGINFO, *LPGETXMLDLGINFO;

#ifdef _WINDOWS_

#define DM_XMLDLGGETINFO      (WM_USER+200)
// wParam - not used
// lParam - [out] LPGETXMLDLGINFO
// return value - true on success

#define DM_XMLDLGSETTEMPLATE  (WM_USER+201)
// wParam - [in] (LPCTSTR) xmldata
// lParam - [in] (DWORD) xmlsize
// this message has no return value

#define DM_XMLDLGWATCHINPUT   (WM_USER+202)
// wParam - [in] (HWND) hwndCursorOver
// lParam - [in] (BOOL) FromMouse; (TRUE - from mouse, FALSE - from set focus)
// this message has no return value

#ifndef APPLOC_NOFUNC

// Fills cx and cy with width and height of the window border (detected from
// style attributes and GetSystemMetrics).
APPLOC_EXTERN void __stdcall XMLGetBorderSize(DWORD dwStyle, DWORD dwExStyle, unsigned int *cx, unsigned int *cy);

// The XMLMapDialogRect function converts (maps) the specified dialog box units
// to screen units (pixels). The function replaces the coordinates in the
// specified RECT structure with the converted coordinates, which allows the
// structure to be used to create a dialog box or position a control within
// a dialog box. 
// This function is used to replace the standard MapDialogRect function,
// because the standard function always uses the default system font.
APPLOC_EXTERN BOOL __stdcall XMLMapDialogRect(HWND hDlg, LPRECT lpRect);

APPLOC_EXTERN int __stdcall XMLTemplateDlgBegin(TCHAR *buffer, int count,
    LPTSTR szTitle, LPTSTR szFontInfo, DWORD dwStyle, DWORD dwExStyle,
    BOOL fScalable, BOOL fNoScroll, BOOL fWatchInput,
    int left, int top, int width, int height);
APPLOC_EXTERN int __stdcall XMLTemplateDlgEnd(TCHAR *buffer, int count);
APPLOC_EXTERN int __stdcall XMLTemplateDlgGroupBegin(TCHAR *buffer, int count, LPTSTR szTitle, int id);
APPLOC_EXTERN int __stdcall XMLTemplateDlgGroupEnd(TCHAR *buffer, int count);
APPLOC_EXTERN int __stdcall XMLTemplateDlgPanelBegin(TCHAR *buffer, int count);
APPLOC_EXTERN int __stdcall XMLTemplateDlgPanelEnd(TCHAR *buffer, int count);
APPLOC_EXTERN int __stdcall XMLTemplateDlgRowBegin(TCHAR *buffer, int count);
APPLOC_EXTERN int __stdcall XMLTemplateDlgRowEnd(TCHAR *buffer, int count);
APPLOC_EXTERN int __stdcall XMLTemplateDlgColBegin(TCHAR *buffer, int count,
    DWORD dwColspan, DWORD dwRowspan, DWORD dwMinWidth, DWORD dwMinHeight,
    DWORD dwLeftMargin, DWORD dwTopMargin, DWORD dwRightMargin, DWORD dwBottomMargin,
    LPTSTR szAlign, LPTSTR szVAlign, DWORD dwStoreRect);
APPLOC_EXTERN int __stdcall XMLTemplateDlgColEnd(TCHAR *buffer, int count);
APPLOC_EXTERN int __stdcall XMLTemplateDlgControl(TCHAR *buffer, int count,
    LPTSTR szType, LPTSTR szTitle, DWORD dwStyle, DWORD dwExStyle, int id,
    BOOL fForceSize, BOOL fForceWidth, BOOL fForceHeight,
    int addwidth, int addheight, double relwidth, double relheight);

#endif // APPLOC_NOFUNC

// hwndParent -     [in] handle to the parent window
// DC -             [in] handle to the device context with font selected
// uID -            [in] child-window identifier
// lpClassName -    [in] pointer to the control class name
// lpCaption -      [in] pointer to the control text
// dwStyle -        [in] control style
// dwExStyle -      [in] extended style
// nMinWidth -      [in] minimal width, in pixels
// nMinHeight -     [in] minimal height, in pixels
// relwidth -       [in] relative width ratio
// relheight -      [in] relative height ratio
// pnWidth -        [out] minimal control width, in pixels
// pnHeight -       [out] minimal control height, in pixels
// pdwStyleUpdate - [out] style update for the specified window class
typedef BOOL (__stdcall *CheckControlSizeExFn)(
    HWND hwndParent, HDC DC, UINT uID, LPCTSTR lpClassName, LPCTSTR lpCaption, 
    DWORD dwStyle, DWORD dwExStyle,
    unsigned int nMinWidth, unsigned int nMinHeight,
    double relwidth, double relheight,
    unsigned int *pnWidth, unsigned int *pnHeight,
    LPDWORD pdwStyleUpdate);

#ifndef APPLOC_NOFUNC

APPLOC_EXTERN LRESULT __stdcall XMLCreateDialogBox(HINSTANCE hInstance,
    HWND hWndParent, DLGPROC lpDialogFunc,
    LPARAM dwInitParam, TCHAR *lpXMLdata,
    DWORD dwXMLsize, BOOL ProcessDialog,
    CheckControlSizeExFn CheckControlSizeEx);

#define XMLDialogBox(hInstance, hWndParent, lpDialogFunc, \
    lpXMLdata, dwXMLsize) \
    (int)XMLCreateDialogBox( \
    hInstance, hWndParent, lpDialogFunc, NULL, lpXMLdata, \
    dwXMLsize, 1, NULL)

#define XMLCreateDialog(hInstance, hWndParent, lpDialogFunc, \
    lpXMLdata, dwXMLsize) \
    (HWND)XMLCreateDialogBox( \
    hInstance, hWndParent, lpDialogFunc, NULL, lpXMLdata, \
    dwXMLsize, 0, NULL)

#define XMLDialogBoxEx(hInstance, hWndParent, lpDialogFunc, \
    lpXMLdata, dwXMLsize, CheckControlSizeEx) \
    (int)XMLCreateDialogBox( \
    hInstance, hWndParent, lpDialogFunc, NULL, lpXMLdata, \
    dwXMLsize, 1, CheckControlSizeEx)

#define XMLCreateDialogEx(hInstance, hWndParent, lpDialogFunc, \
    lpXMLdata, dwXMLsize, CheckControlSizeEx) \
    (HWND)XMLCreateDialogBox( \
    hInstance, hWndParent, lpDialogFunc, NULL, lpXMLdata, \
    dwXMLsize, 0, CheckControlSizeEx)

#define XMLDialogBoxParam(hInstance, hWndParent, lpDialogFunc, \
    dwInitParam, lpXMLdata, dwXMLsize) \
    (int)XMLCreateDialogBox( \
    hInstance, hWndParent, lpDialogFunc, dwInitParam, lpXMLdata, \
    dwXMLsize, 1, NULL)

#define XMLCreateDialogParam(hInstance, hWndParent, lpDialogFunc, \
    dwInitParam, lpXMLdata, dwXMLsize) \
    (HWND)XMLCreateDialogBox( \
    hInstance, hWndParent, lpDialogFunc, dwInitParam, lpXMLdata, \
    dwXMLsize, 0, NULL)

#define XMLDialogBoxParamEx(hInstance, hWndParent, lpDialogFunc, \
    dwInitParam, lpXMLdata, dwXMLsize, CheckControlSizeEx) \
    (int)XMLCreateDialogBox( \
    hInstance, hWndParent, lpDialogFunc, dwInitParam, lpXMLdata, \
    dwXMLsize, 1, CheckControlSizeEx)

#define XMLCreateDialogParamEx(hInstance, hWndParent, lpDialogFunc, \
    dwInitParam, lpXMLdata, dwXMLsize, CheckControlSizeEx) \
    (HWND)XMLCreateDialogBox( \
    hInstance, hWndParent, lpDialogFunc, dwInitParam, lpXMLdata, \
    dwXMLsize, 0, CheckControlSizeEx)

#endif // APPLOC_NOFUNC

#endif // _WINDOWS_

#ifdef __cplusplus
}
#endif

#ifdef _MSC_VER 

#include <poppack.h>

#endif // _MSC_VER 

#ifdef __BORLANDC__

#pragma option -b.
#include <poppack.h>
#pragma option -b

#endif // __BORLANDC__

#endif // XMLDLG_H_
