/*****************************************************************************
 *
 *   (C) Copyright Southsoftware.com, 2005
 *
 *   Title: XMLPS.H
 *
 *   Version:   1.09
 *
 *   Date:  28-Jul-2009
 *
 *   Author:    Master
 *
 *----------------------------------------------------------------------------
 *
 *   Change log:
 *
 *   DATE        DESCRIPTION
 *   ----------- -------------------------------------------------------------
 *   19-Sep-2005 Original
 *   28-Jul-2009 Last edition
 */

#ifndef XMLPS_H_
#define XMLPS_H_ 1

///////////////////////////////////////////////////////////////////////////////
// Decorate exportable functions for Win32 DLL linking.
#if (defined(WIN32) || defined(_WIN32)) && !defined(APPLOC_STATICLIB)
#if defined(APPLOC_BUILDDLL)
#define APPLOC_EXTERN  __declspec(dllexport)
#else
#define APPLOC_EXTERN  __declspec(dllimport)
#endif
#else
#define APPLOC_EXTERN
#endif

///////////////////////////////////////////////////////////////////////////////
#ifdef _MSC_VER
// Assume 4 byte packing throughout.
#include <pshpack4.h>
#elif __BORLANDC__
// Turn off "Structure packing size has changed" warning message.
#pragma option -w-pck
// Assume 4 byte packing throughout.
#pragma option -b.
#include <pshpack4.h>
#pragma option -b
#else
#error You must 4-byte align these structures with the appropriate compiler directives.
#endif

#ifdef __cplusplus
extern "C" {
#endif

///////////////////////////////////////////////////////////////////////////////
// Chooses the Back button.
#define XMLPSBTN_BACK              0
// Chooses the Next button.
#define XMLPSBTN_NEXT              1
// Chooses the Finish button.
#define XMLPSBTN_FINISH            2
// Chooses the OK button.
#define XMLPSBTN_OK                3
// Chooses the Apply Now button.
#define XMLPSBTN_APPLYNOW          4
// Chooses the Cancel button.
#define XMLPSBTN_CANCEL            5
// Chooses the Help button.
#define XMLPSBTN_HELP              6
// Chooses the Buy Now button.
#define XMLPSBTN_BUYNOW            7
// Chooses the Close button.
#define XMLPSBTN_CLOSE             8

typedef struct tagXMLPSNQUERYCANCEL {
    NMHDR hdr;
    BOOL bButton; // TRUE if Cancel button is pressed.
                  // FALSE if something like [X] is pressed.
} XMLPSNQUERYCANCEL, FAR *LPXMLPSNQUERYCANCEL;

typedef struct tagXMLPSNWATCHINPUT {
    NMHDR hdr;
    INT cursor;  // Index of a button (XMLPSBTN_ constants).
} XMLPSNWATCHINPUT, FAR *LPXMLPSNWATCHINPUT;


#define XMLPSN_FIRST               (0U-200U)
#define XMLPSN_LAST                (0U-299U)


#define XMLPSN_SETACTIVE           (XMLPSN_FIRST-0)
#define XMLPSN_KILLACTIVE          (XMLPSN_FIRST-1)
#define XMLPSN_APPLY               (XMLPSN_FIRST-2)
#define XMLPSN_RESET               (XMLPSN_FIRST-3)
#define XMLPSN_HELP                (XMLPSN_FIRST-4)
#define XMLPSN_WIZBACK             (XMLPSN_FIRST-5)
#define XMLPSN_WIZNEXT             (XMLPSN_FIRST-6)
#define XMLPSN_WIZFINISH           (XMLPSN_FIRST-7)
#define XMLPSN_QUERYCANCEL         (XMLPSN_FIRST-8)
#define XMLPSN_BUYNOW              (XMLPSN_FIRST-9)
#define XMLPSN_WATCHINPUT          (XMLPSN_FIRST-10)

#define XMLPSNRET_NOERROR              0
#define XMLPSNRET_INVALID              1
#define XMLPSNRET_INVALID_NOCHANGEPAGE 2


#define XMLPSM_SETCURSELHWND        (WM_USER + 101)
// wParam - [in] (HWND) hwnd - Handle to a page.
// lParam - not used
// Returns TRUE on success.
#define XMLPropSheet_SetCurSelHwnd(hDlg, hwnd) \
        (BOOL)SendMessage(hDlg, XMLPSM_SETCURSELHWND, (WPARAM)hwnd, (LPARAM)NULL)


#define XMLPSM_SETCURSELID          (WM_USER + 102)
// wParam - [in] (int) index - Index of a page.
// lParam - not used
// Returns TRUE on success.
#define XMLPropSheet_SetCurSelId(hDlg, index) \
        (BOOL)SendMessage(hDlg, XMLPSM_SETCURSELID, (WPARAM)index, (LPARAM)NULL)


#define XMLPSM_ADDPAGE             (WM_USER + 103)
// wParam - [in] (int) index - Index of a page after which the new page should be inserted.
// lParam - [in] (LPXMLPROPERTYSHEETPAGE)lpXMLpsp
// Returns TRUE on success (GetLastError() can be used for extended error information).
#define XMLPropSheet_AddPage(hDlg, index, lpXMLpsp) \
        (BOOL)SendMessage(hDlg, XMLPSM_ADDPAGE, (WPARAM)index, (LPARAM)lpXMLpsp)


#define XMLPSM_REMOVEPAGE          (WM_USER + 104)
// wParam - [in] (int) index - Index of a page to remove.
// lParam - not used
// Returns TRUE on success.
#define XMLPropSheet_RemovePage(hDlg, index) \
        (BOOL)SendMessage(hDlg, XMLPSM_REMOVEPAGE, index, (LPARAM)NULL)


#define XMLPSM_CHANGED             (WM_USER + 105)
// wParam - [in] (HWND) hwndPage - Handle to the page that has changed.
// lParam - not used
// Returns TRUE on success.
#define XMLPropSheet_Changed(hDlg, hwndPage) \
        (BOOL)SendMessage(hDlg, XMLPSM_CHANGED, (WPARAM)hwndPage, 0L)


#define XMLPSM_CANCELTOCLOSE       (WM_USER + 106)
// wParam - not used
// lParam - not used
// Returns TRUE on success.
#define XMLPropSheet_CancelToClose(hDlg) \
        PostMessage(hDlg, XMLPSM_CANCELTOCLOSE, 0, 0L)


#define XMLPSM_UNCHANGED           (WM_USER + 107)
// wParam - [in] (HWND) hwndPage - Handle to the page that has reverted to the previously saved state.
// lParam - not used
// Returns TRUE on success.
#define XMLPropSheet_UnChanged(hDlg, hwndPage) \
        (BOOL)SendMessage(hDlg, XMLPSM_UNCHANGED, (WPARAM)hwndPage, 0L)


#define XMLPSM_APPLY               (WM_USER + 108)
// wParam - not used
// lParam - not used
// Returns TRUE on success.
#define XMLPropSheet_Apply(hDlg) \
        (BOOL)SendMessage(hDlg, XMLPSM_APPLY, 0, 0L)


#define XMLPSM_SETTITLEA           (WM_USER + 109)
#define XMLPSM_SETTITLEW           (WM_USER + 110)
// wParam - not used
// lParam - [in] (LPCTSTR) lpszText - Pointer to a buffer that contains the title string.
// Returns TRUE on success.

#ifdef UNICODE
#define XMLPSM_SETTITLE            XMLPSM_SETTITLEW
#else
#define XMLPSM_SETTITLE            XMLPSM_SETTITLEA
#endif

#define XMLPropSheet_SetTitle(hDlg, lpszText)\
        (BOOL)SendMessage(hDlg, XMLPSM_SETTITLE, 0, (LPARAM)(LPCTSTR)lpszText)


#define XMLPSM_SETWIZBUTTONS       (WM_USER + 111)
// wParam - not used
// lParam - [in] (DWORD) dwFlags - Specifies the buttons to display and enable.
// Returns TRUE on success.
#define XMLPropSheet_SetWizButtons(hDlg, dwFlags) \
        PostMessage(hDlg, XMLPSM_SETWIZBUTTONS, 0, (LPARAM)dwFlags)
// Enables the Back button.
#define XMLPSWIZB_BACK             0x00000001
// Enables the Next button.
#define XMLPSWIZB_NEXT             0x00000002
// Displays an enabled Finish button in place of the Next button.
#define XMLPSWIZB_FINISH           0x00000004
// Displays a disabled Finish button in place of the Next button.
#define XMLPSWIZB_DISABLEDFINISH   0x00000008


#define XMLPSM_PRESSBUTTON         (WM_USER + 112)
// wParam - [in] (INT) iButton - Index of the button to choose (XMLPSBTN_ constants).
// lParam - not used
// Returns TRUE on success.
#define XMLPropSheet_PressButton(hDlg, iButton) \
        PostMessage(hDlg, XMLPSM_PRESSBUTTON, (WPARAM)iButton, 0L)
 

#define XMLPSM_SETWIZARDBTNTEXT    (WM_USER + 113)
// wParam - [in] (DWORD) iButton - Index of the button to change text (XMLPSWIZBTN_ constants).
// lParam - [in] (LPCTSTR) lpszText
// Returns TRUE on success.
#define XMLPropSheet_SetWizardBtnText(hDlg, iButton, lpszText) \
        (BOOL)SendMessage(hDlg, XMLPSM_SETWIZARDBTNTEXT, (WPARAM)iButton, (LPARAM)lpszText)
// Changes text on the Back button.
#define XMLPSWIZBTN_BACK              0
// Changes text on the Next button.
#define XMLPSWIZBTN_NEXT              1
// Changes text on the Finish button.
#define XMLPSWIZBTN_FINISH            2
// Changes text on the Cancel button.
#define XMLPSWIZBTN_CANCEL            3


#define XMLPSM_SETSTATUSTEXT       (WM_USER + 114)
// wParam - not used
// lParam - [in] lpszText
// Returns TRUE on success.
#define XMLPropSheet_SetStatusText(hDlg, lpszText) \
        (BOOL)SendMessage(hDlg, XMLPSM_SETSTATUSTEXT, 0, (LPARAM)lpszText)


#define XMLPSM_GETTABCONTROL       (WM_USER + 115)
// wParam - not used
// lParam - not used
// Returns the handle to the tab control.
#define XMLPropSheet_GetTabControl(hDlg) \
        (HWND)SendMessage(hDlg, XMLPSM_GETTABCONTROL, 0, 0L)


#define XMLPSM_GETTREECONTROL       (WM_USER + 116)
// wParam - not used
// lParam - not used
// Returns the handle to the tree view control.
#define XMLPropSheet_GetTreeControl(hDlg) \
        (HWND)SendMessage(hDlg, XMLPSM_GETTREECONTROL, 0, 0L)


#define XMLPSM_ISDIALOGMESSAGE     (WM_USER + 117)
// wParam - not used
// lParam - [in] pMsg - Pointer to a MSG structure that contains the message to be checked.
// If the message has been processed, the return value is TRUE.
#define XMLPropSheet_IsDialogMessage(hDlg, pMsg) \
        (BOOL)SendMessage(hDlg, XMLPSM_ISDIALOGMESSAGE, 0, (LPARAM)pMsg)


#define XMLPSM_GETCURRENTPAGEHWND  (WM_USER + 118)
// wParam - not used
// lParam - not used
// Returns a handle to the window of the current property sheet page.
#define XMLPropSheet_GetCurrentPageHwnd(hDlg) \
        (HWND)SendMessage(hDlg, XMLPSM_GETCURRENTPAGEHWND, 0, 0L)


#define XMLPSM_GETCURRENTPAGEID  (WM_USER + 119)
// wParam - not used
// lParam - not used
// Returns an index of the current property sheet page.
#define XMLPropSheet_GetCurrentPageId(hDlg) \
        (int)SendMessage(hDlg, XMLPSM_GETCURRENTPAGEID, 0, 0L)


#define XMLPSM_GETPAGECOUNT  (WM_USER + 120)
// wParam - not used
// lParam - not used
// Returns number of pages in a property sheet.
#define XMLPropSheet_GetPageCount(hDlg) \
        (int)SendMessage(hDlg, XMLPSM_GETPAGECOUNT, 0, 0L)


typedef struct tagXMLPSSETFONT {
    HFONT hFont;
    BOOL fRedraw;
} XMLPSSETFONT, FAR *LPXMLPSSETFONT;

#define XMLPSM_SETFONT (WM_USER + 121)
// wParam - [in] (int) index of the page to change font.
// lParam - [in] LPXMLPSSETFONT
// Returns TRUE on success.
#define XMLPropSheet_SetFont(hDlg, index, lpxmlsf) \
        (BOOL)SendMessage(hDlg, XMLPSM_SETFONT, index, (LPARAM)lpxmlsf)


typedef struct tagXMLPSSETTEMPLATE {
    TCHAR *xmldata;
    DWORD xmlsize;
} XMLPSSETTEMPLATE, FAR *LPXMLPSSETTEMPLATE;

#define XMLPSM_SETTEMPLATE  (WM_USER + 122)
// wParam - [in] (int) index of the page to change template.
// lParam - [in] LPXMLPSSETTEMPLATE
// Returns TRUE on success.
#define XMLPropSheet_SetTemplate(hDlg, index, lpxmlst) \
        (BOOL)SendMessage(hDlg, XMLPSM_SETTEMPLATE, index, (LPARAM)lpxmlst)


#define XMLPSM_SETXMLPROPERTYTEXT  (WM_USER + 123)
// wParam - [in] not used
// lParam - [in] LPXMLPROPERTYTEXT
// Returns TRUE on success.
#define XMLPropSheet_SetXMLPropertyText(hDlg, lpxmlpt) \
        (BOOL)SendMessage(hDlg, XMLPSM_SETXMLPROPERTYTEXT, 0, (LPARAM)lpxmlpt)


#define XMLPSM_REMOVEBUYNOWBUTTON  (WM_USER + 124)
// wParam - [in] not used
// lParam - [in] not used
// Returns TRUE on success.
#define XMLPropSheet_RemoveBuyNowButton(hDlg) \
        (BOOL)SendMessage(hDlg, XMLPSM_REMOVEBUYNOWBUTTON, 0, 0L)


#define XMLPSM_SELECTBUTTON       (WM_USER + 125)
// wParam - [in] (INT) iButton - Index of the button to choose (XMLPSBTN_ constants).
// lParam - not used
// Returns TRUE on success.
#define XMLPropSheet_SelectButton(hDlg, iButton) \
        PostMessage(hDlg, XMLPSM_SELECTBUTTON, (WPARAM)iButton, 0L)


#define XMLPS_DEFAULT             0x00000000
#define XMLPS_USESIZE             0x00000008
#define XMLPS_USESIZEMIN          0x00000010
#define XMLPS_NOAPPLYNOW          0x00000020
#define XMLPS_NOBUYNOW            0x00000040
#define XMLPS_NOCANCEL            0x00000080
#define XMLPS_HASHELP             0x00000100
#define XMLPS_HASSTATUSBAR        0x00000200
#define XMLPS_MODELESS            0x00000400
#define XMLPS_NODISABLEPARENT     0x00000800
#define XMLPS_WIZARD              0x00001000
#define XMLPS_RESIZABLE           0x00002000
#define XMLPS_RTLREADING          0x00004000
#define XMLPS_CONTEXTHELP         0x00008000
#define XMLPS_CENTERSCREEN        0x00010000
#define XMLPS_MAXIMIZEBOX         0x00020000
#define XMLPS_MINIMIZEBOX         0x00040000
#define XMLPS_TOPMOST             0x00080000
#define XMLPS_NOCLOSE             0x00100000
#define XMLPS_APPLYONCHANGE       0x00200000

typedef struct tagXMLPROPERTYTEXT {
    LPCTSTR    pszOK;        // Text of the "OK" button.
    LPCTSTR    pszCancel;    // Text of the "Cancel" button.
    LPCTSTR    pszClose;     // Text of the "Close" button.
    LPCTSTR    pszApplyNow;  // Text of the "Apply Now" button.
    LPCTSTR    pszBuyNow;    // Text of the "Buy Now" button.
    LPCTSTR    pszHelp;      // Text of the "Help" button.
    LPCTSTR    pszBack;      // Text of the "Back" button.
    LPCTSTR    pszNext;      // Text of the "Next" button.
    LPCTSTR    pszFinish;    // Text of the "Finish" button.
} XMLPROPERTYTEXT, FAR *LPXMLPROPERTYTEXT;

#define XMLPS_CALLBACKBUTTON 0
#define XMLPS_CALLBACKWIZTOP 1
#define XMLPS_CALLBACKWIZERASE 2
#define XMLPS_CALLBACKWIZDRAW 3
typedef BOOL (__stdcall *XMLPSCallbackFn)(HWND hwnd, int iMsg, WPARAM wParam, LPARAM lParam);

// Property sheet creation structure.
// Only the following flags are available in this structure:
// XMLPS_DEFAULT         - Uses the default meaning for all structure members.
// XMLPS_USESIZE         - Uses cx and cy as the initial size of a property sheet window. Otherwise the size will be taken from the largest page dialog.
// XMLPS_NOAPPLYNOW      - Removes the Apply Now button.
// XMLPS_NOBUYNOW        - Removes the Buy Now button.
// XMLPS_NOCANCEL        = Removes Cancel button.
// XMLPS_HASHELP         - Displays the property sheet Help button.
// XMLPS_HASSTATUSBAR    - Displays the status bar.
// XMLPS_MODELESS        - Creates the property sheet as a modeless dialog.
// XMLPS_NODISABLEPARENT - Do not disable parent window.
// XMLPS_WIZARD          - Creates a wizard property sheet.
// XMLPS_RESIZABLE       - Creates the property sheet with resizable window frame.
// XMLPS_RTLREADING      - Displays the title of the property sheet dialog using right-to-left reading order.
// XMLPS_CONTEXTHELP     - Displays a question mark in the title bar of the window.
// XMLPS_CENTERSCREEN    - Displays property sheet window in the center of the screen.
// XMLPS_MAXIMIZEBOX     - Creates a window that has a Maximize button. Cannot be combined with the XMLPS_CONTEXTHELP style.
// XMLPS_MINIMIZEBOX     - Creates a window that has a Minimize button. Cannot be combined with the XMLPS_CONTEXTHELP style.
// XMLPS_TOPMOST         - Specifies that a window created with this style should be placed above all non-topmost windows and should stay above them, even when the window is deactivated.
// XMLPS_NOCLOSE         - Disables the Close command on the System menu.
// XMLPS_APPLYONCHANGE   - Send XMLPSN_APPLY notification message when property page is about to change
typedef struct tagXMLPROPERTYSHEET {
    DWORD      dwFlags;       // A set of bit flags (XMLPS_ constants).
    HWND       hwndParent;    // Handle of the owner window.
    HINSTANCE  hInstance;     // Handle of the instance from which to load the icon resource.
    HICON      hIcon;         // Handle of the icon of the property sheet dialog box.
    HICON      hIconSm;       // Handle of the icon to use as the small icon in the title bar of the property sheet dialog box.
    LPCTSTR    pszCaption;    // Title of the property sheet dialog box.
    LPCTSTR    pszFontInfo;   // Info of the font of property sheet dialog box.
    UINT       x, y;          // Specifies coordinates, in dialog box units, of the upper-left corner of the dialog box.
    UINT       cx, cy;        // Specifies the width and height, in dialog box units, of the dialog box.
    XMLPROPERTYTEXT propText; // Text strings for property sheet buttons.
    XMLPSCallbackFn XMLPSCallback; // Callback function for property sheet.
} XMLPROPERTYSHEET, FAR *LPXMLPROPERTYSHEET;

// Property page creation structure.
typedef struct tagXMLPROPERTYSHEETPAGE {
    HINSTANCE hInstance;      // Handle to the instance from which to load the dialog box template, icon, or title string resource.
    LPCTSTR   lpXMLdata;      // Identifies dialog box XML template.
    DWORD     dwXMLsize;      // Size, in characters, of the dialog box XML template.
    CheckControlSizeExFn CheckControlSizeEx; // Pointer to the user-defined callback function.
    HICON     hIcon;          // Handle of the icon to use as the small icon in the title bar of the property sheet dialog box.
    LPCTSTR   pszIcon;        // Icon resource to use as the small icon in the title bar of the property sheet dialog box.
    DLGPROC   pfnDlgProc;     // Pointer to the dialog box procedure for the page. The dialog box procedure must not call the EndDialog function.
    LPARAM    lParam;         // Application-defined data.
} XMLPROPERTYSHEETPAGE, FAR *LPXMLPROPERTYSHEETPAGE;

#ifndef APPLOC_NOFUNC
APPLOC_EXTERN HANDLE __stdcall XMLPropertySheetCreate(LPXMLPROPERTYSHEET lpXMLpss);
APPLOC_EXTERN BOOL __stdcall XMLPropertySheetAdd(HANDLE hXMLpss, LPXMLPROPERTYSHEETPAGE lpXMLpsp);
APPLOC_EXTERN LRESULT __stdcall XMLPropertySheetShow(HANDLE hXMLpss, UINT  nStartPage);
APPLOC_EXTERN int __stdcall XMLPropertySheetLastResult(HANDLE hXMLpss);
APPLOC_EXTERN BOOL __stdcall XMLPropertySheetFree(HANDLE hXMLpss);
#endif // APPLOC_NOFUNC

#ifdef __cplusplus
}
#endif

#ifdef _MSC_VER 

#include <poppack.h>

#endif // _MSC_VER 

#ifdef __BORLANDC__

#pragma option -b.
#include <poppack.h>
#pragma option -b

#endif // __BORLANDC__

#endif // XMLPS_H_
